-- SpamageMeters - an addon to suppress annoying Recount damage output stuff and 
-- instead condense it into clickable chat links
--
-- Written by Wrug and Cybey
--
-- Code inspiration from the WelcomeHome tutorial, Deadly Boss Mods and GreedBeacon

SpamageMeters = LibStub("AceAddon-3.0"):NewAddon("SpamageMeters", "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0", "AceTimer-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("SpamageMeters")

local defaults = {
    profile =  {
	filterGuild = 2,
	filterOfficer = 2,
	filterParty = 2,
	filterRaid = 2,
	filterSay = 2,
	filterWhisper = 2,
	filterYell = 2,
	debug = false,
	captureDelay = 1.0,
    },
}

local spamFirstLines = {
	"^Recount - (.*)$",
}

local spamNextLines = {
	"^(%d+). (.*)$",
}

local meters = {}


local options = {
	name = "SpamageMeters",
	handler = SpamageMeters,
	type = "group",
	args = {
--		debug = {
--			type = "toggle",
--			name = L["Enable Debug"],
--			desc = L["Enable the debug function"],
--			set = function(info, value)
--				SpamageMeters.db.profile.debug = value
--			end,
--			get = function(info)
--				return SpamageMeters.db.profile.debug
--			end
--		},
        	filterGuild = {
			type = "select",
			name = L["Filter Guild"],
			desc = L["Selects the action to perform when encountering damage meter data in guild chat"],
			set = function(info, value)
				SpamageMeters.db.profile.filterGuild = value
			end,
			get = function(info)
				return SpamageMeters.db.profile.filterGuild
			end,
			values = {	
						[1] = L["Do Nothing"],
						[2] = L["Compress"],
						[3] = L["Suppress"]
					}
		},
        	filterOfficer = {
			type = "select",
			name = L["Filter Officer"],
			desc = L["Selects the action to perform when encountering damage meter data in officer chat"],
			set = function(info, value)
				SpamageMeters.db.profile.filterOfficer = value
			end,
			get = function(info)
				return SpamageMeters.db.profile.filterOfficer
			end,
			values = {	
						[1] = L["Do Nothing"],
						[2] = L["Compress"],
						[3] = L["Suppress"]
					}
		},
        	filterParty = {
			type = "select",
			name = L["Filter Party"],
			desc = L["Selects the action to perform when encountering damage meter data in party chat"],
			set = function(info, value)
				SpamageMeters.db.profile.filterParty = value
			end,
			get = function(info)
				return SpamageMeters.db.profile.filterParty
			end,
			values = {	
						[1] = L["Do Nothing"],
						[2] = L["Compress"],
						[3] = L["Suppress"]
					}
		},
        	filterRaid = {
			type = "select",
			name = L["Filter Raid"],
			desc = L["Selects the action to perform when encountering damage meter data in raid chat"],
			set = function(info, value)
				SpamageMeters.db.profile.filterRaid = value
			end,
			get = function(info)
				return SpamageMeters.db.profile.filterRaid
			end,
			values = {	
						[1] = L["Do Nothing"],
						[2] = L["Compress"],
						[3] = L["Suppress"]
					}
		},
        	filterSay = {
			type = "select",
			name = L["Filter Say"],
			desc = L["Selects the action to perform when encountering damage meter data in say chat"],
			set = function(info, value)
				SpamageMeters.db.profile.filterSay = value
			end,
			get = function(info)
				return SpamageMeters.db.profile.filterSay
			end,
			values = {	
						[1] = L["Do Nothing"],
						[2] = L["Compress"],
						[3] = L["Suppress"]
					}
		},
        	filterWhisper = {
			type = "select",
			name = L["Filter Whisper"],
			desc = L["Selects the action to perform when encountering damage meter whisper"],
			set = function(info, value)
				SpamageMeters.db.profile.filterWhisper = value
			end,
			get = function(info)
				return SpamageMeters.db.profile.filterWhisper
			end,
			values = {	
						[1] = L["Do Nothing"],
						[2] = L["Compress"],
						[3] = L["Suppress"]
					}
		},
        	filterYell = {
			type = "select",
			name = L["Filter Yell"],
			desc = L["Selects the action to perform when encountering damage meter data in yell chat"],
			set = function(info, value)
				SpamageMeters.db.profile.filterYell = value
			end,
			get = function(info)
				return SpamageMeters.db.profile.filterYell
			end,
			values = {	
						[1] = L["Do Nothing"],
						[2] = L["Compress"],
						[3] = L["Suppress"]
					}
		},
			captureDelay = {
			type = "range",
			name = L["Capture Delay"],
			desc = L["How many seconds the addon waits after 'Recount - *' lines before it assumes spam burst is over. 1 seems to work in most cases"],
			set = function(info, value)
				SpamageMeters.db.profile.captureDelay = value
			end,
			get = function(info)
				return SpamageMeters.db.profile.captureDelay
			end,
			min = 1,
			max = 5,
			step = 0.1
		},
	},
}

function SpamageMeters:OnInitialize()
	-- Called when the addon is loaded
	self.db = LibStub("AceDB-3.0"):New("SpamageMetersDB", defaults, "Default")

	LibStub("AceConfig-3.0"):RegisterOptionsTable("SpamageMeters", options)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("SpamageMeters", "SpamageMeters")
	self:RegisterChatCommand("spm", "ChatCommand")
	self:RegisterChatCommand("spamagemeters", "ChatCommand")
end

function SpamageMeters:OnEnable()
	-- Prat workaround
	if IsAddOnLoaded("Prat-3.0") then
		self:ScheduleTimer("PostEnable", 2)
	else
		self:ScheduleTimer("PostEnable", 0)
	end
end


function SpamageMeters:PostEnable()
	self:RawHook("ChatFrame_MessageEventHandler","ParseChatEvent", true)
	self:RawHook("SetItemRef","ParseLink",true)
end

function SpamageMeters:OnDisable()
	-- Called when the addon is disabled
end

function SpamageMeters:Debug(...)
	-- Print the message to screen if debug is turned on
	if self.db.profile.debug then
		self:Print(...)
	end
end

function SpamageMeters:ChatCommand(input)
	-- Open dialog if no parameters, else do something with the parameters
	if not input or input:trim() == "" then
		InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
	else
		LibStub("AceConfigCmd-3.0").HandleCommand(SpamageMeters, "spm", "SpamageMeters", input)
	end
end

function SpamageMeters:ParseLink(link, text, button)
	local linktype, id = strsplit(":", link)
	if linktype == "SPM" then
		local meterID = tonumber(id)
		-- put stuff in the ItemRefTooltip from FrameXML
		ShowUIPanel(ItemRefTooltip);
		if ( not ItemRefTooltip:IsShown() ) then
			ItemRefTooltip:SetOwner(UIParent, "ANCHOR_PRESERVE");
		end
		ItemRefTooltip:ClearLines()
		ItemRefTooltip:AddLine(meters[meterID].title)
		ItemRefTooltip:AddLine(string.format(L["Reported by %s"],meters[meterID].src))
		for k,v in ipairs(meters[meterID].data) do
			--local left,right = strsplit("  ", v)
			--ItemRefTooltip:AddDoubleLine(left,right)
			ItemRefTooltip:AddLine(v,1,1,1)
		end
		ItemRefTooltip:Show()
		
		
		
		
	else
		return self.hooks["SetItemRef"](link, text, button)
	end
end


function SpamageMeters:filterLine(event, source, msg)
	local isSpam = false
	for k,v in ipairs(spamNextLines) do
		if msg:match(v) then
			self:Debug(msg,"matched",v)
			for i,j in ipairs(meters) do
				local curTime = GetTime()
				local elapsed = curTime - j.time
				if j.src == source and j.evt == event and elapsed < self.db.profile.captureDelay then
					table.insert(j.data,msg)
					return true, false, nil
				end
			end
		end
	end
	
	for k,v in ipairs(spamFirstLines) do
		--if string.find(string.lower(msg), string.lower(v)) ~= nil then
		local newID = 0
		if msg:match(v) then
			local curTime = GetTime();
			local newMeter = {src = source, evt = event, time = curTime, data = {}, title = msg}
			table.insert(meters, newMeter)
			
			for i,j in ipairs(meters) do
				if j.src == source and j.evt == event and j.time == curTime then
					newID = i
				end
			end

			return true, true, string.format("SpamageMeters: |HSPM:%1$d|h|cFFFFFF00[%2$s]|r|h",newID or 0,msg or "nil")
		end
	end
	return false, false, nil
end


function SpamageMeters:ParseChatEvent(this, event, msg, sender, ...)
	local hide = false

	-- When getting a guild chat, evaluate
	if event == "CHAT_MSG_GUILD" and self.db.profile.filterGuild > 1 then
		isRecount, isFirstLine, newMessage = self:filterLine(event, sender, msg)
		if isRecount then
			if isFirstLine and self.db.profile.filterGuild == 2 then
				local debugmsg = gsub(newMessage, "\124", "\124\124");
				self:Debug(msg,"changed to",debugmsg)
				msg = newMessage
			else
				-- it's not a first line or it's set to suppress anyway, hide the stuff
				self:Debug(msg,"suppressed")
				hide = true
			end
		end
	end
	
	-- When getting an officer chat, evaluate
	if event == "CHAT_MSG_OFFICER" and self.db.profile.filterOfficer > 1 then
		isRecount, isFirstLine, newMessage = self:filterLine(event, sender, msg)
		if isRecount then
			if isFirstLine and self.db.profile.filterOfficer == 2 then
				local debugmsg = gsub(newMessage, "\124", "\124\124");
				self:Debug(msg,"changed to",debugmsg)
				msg = newMessage
			else
				-- it's not a first line or it's set to suppress anyway, hide the stuff
				self:Debug(msg,"suppressed")
				hide = true
			end
		end
	end
	
	-- When getting a party chat, evaluate
	if event == "CHAT_MSG_PARTY" and self.db.profile.filterParty > 1 then
		isRecount, isFirstLine, newMessage = self:filterLine(event, sender, msg)
		if isRecount then
			if isFirstLine and self.db.profile.filterParty == 2 then
				local debugmsg = gsub(newMessage, "\124", "\124\124");
				self:Debug(msg,"changed to",debugmsg)
				msg = newMessage
			else
				-- it's not a first line or it's set to suppress anyway, hide the stuff
				self:Debug(msg,"suppressed")
				hide = true
			end
		end
	end

	-- When getting a raid chat, evaluate
	if event == "CHAT_MSG_RAID" and self.db.profile.filterRaid > 1 then
		isRecount, isFirstLine, newMessage = self:filterLine(event, sender, msg)
		if isRecount then
			if isFirstLine and self.db.profile.filterRaid == 2 then
				local debugmsg = gsub(newMessage, "\124", "\124\124");
				self:Debug(msg,"changed to",debugmsg)
				msg = newMessage
			else
				-- it's not a first line or it's set to suppress anyway, hide the stuff
				self:Debug(msg,"suppressed")
				hide = true
			end
		end
	end
	
	-- When getting a say chat, evaluate
	if event == "CHAT_MSG_SAY" and self.db.profile.filterSay > 1 then
		isRecount, isFirstLine, newMessage = self:filterLine(event, sender, msg)
		if isRecount then
			if isFirstLine and self.db.profile.filterSay == 2 then
				local debugmsg = gsub(newMessage, "\124", "\124\124");
				self:Debug(msg,"changed to",debugmsg)
				msg = newMessage
			else
				-- it's not a first line or it's set to suppress anyway, hide the stuff
				self:Debug(msg,"suppressed")
				hide = true
			end
		end
	end
	
	-- When getting a whisper, evaluate
	if event == "CHAT_MSG_WHISPER" and self.db.profile.filterWhisper > 1 then
		isRecount, isFirstLine, newMessage = self:filterLine(event, sender, msg)
		if isRecount then
			if isFirstLine and self.db.profile.filterWhisper == 2 then
				local debugmsg = gsub(newMessage, "\124", "\124\124");
				self:Debug(msg,"changed to",debugmsg)
				msg = newMessage
			else
				-- it's not a first line or it's set to suppress anyway, hide the stuff
				self:Debug(msg,"suppressed")
				hide = true
			end
		end
	end
	
	-- When getting a yell, evaluate
	if event == "CHAT_MSG_YELL" and self.db.profile.filterYell > 1 then
		isRecount, isFirstLine, newMessage = self:filterLine(event, sender, msg)
		if isRecount then
			if isFirstLine and self.db.profile.filterYell == 2 then
				local debugmsg = gsub(newMessage, "\124", "\124\124");
				self:Debug(msg,"changed to",debugmsg)
				msg = newMessage
			else
				-- it's not a first line or it's set to suppress anyway, hide the stuff
				self:Debug(msg,"suppressed")
				hide = true
			end
		end
	end

	
	if not hide then
		return self.hooks["ChatFrame_MessageEventHandler"](this, event, msg, sender, ...)
	end
	return
end
